variable "INSTANCES_COUNT" {
  type        = number
  description = "Number of instances to create"
}

variable "PROJECT_ID" {
  type        = string
  description = "Project ID"
}

terraform {
  required_providers {
    scaleway = {
      source = "scaleway/scaleway"
    }
  }
  required_version = ">= 0.13"
}

provider "scaleway" {
  zone   = "fr-par-1"
  region = "fr-par"
}


resource "scaleway_instance_ip" "public_ip" {
  count      = var.INSTANCES_COUNT
  project_id = var.PROJECT_ID
}

resource "scaleway_instance_security_group" "www" {
  project_id              = var.PROJECT_ID
  inbound_default_policy  = "drop"
  outbound_default_policy = "accept"

  inbound_rule {
    action = "accept"
    port   = "22"
  }

  inbound_rule {
    action = "accept"
    port   = "80"
  }

  inbound_rule {
    action = "accept"
    port   = "8080"
  }

  inbound_rule {
    action = "accept"
    port   = "443"
  }
}

resource "scaleway_instance_server" "test" {
  count      = var.INSTANCES_COUNT
  project_id = var.PROJECT_ID
  type       = "DEV1-L"
  image      = "ubuntu_focal"

  tags  = ["test-destroy"]
  ip_id = scaleway_instance_ip.public_ip[count.index].id

  security_group_id = scaleway_instance_security_group.www.id
}

output "public_ip_address" {
  value = scaleway_instance_ip.public_ip
}

resource "null_resource" "provisioner" {
  count = var.INSTANCES_COUNT

  connection {
    host     = element(scaleway_instance_ip.public_ip.*.address, count.index)
    type     = "ssh"
    user     = "root"
    password = ""
  }

  provisioner "file" {
    source      = "./instance_scripts"
    destination = "/root"
  }

  // Install docker and webtop
  provisioner "remote-exec" {
    inline = [
      "curl -fsSL https://get.docker.com -o get-docker.sh",
      "sudo sh get-docker.sh",
      "chmod 770 /root/instance_scripts/*",
      "/root/instance_scripts/start_webtop.sh",
      "docker exec -it webtop bash -c 'echo -e \"yfB4W23G\nyfB4W23G\" | passwd abc'"
    ]
  }

  // Install kali packages
  provisioner "remote-exec" {
    inline = [
      "docker exec -it webtop bash -c 'apt update -y'",
      "docker exec -it webtop bash -c 'apt install openvpn git nano -y'",
      "docker exec -it webtop bash -c 'apt install nmap john sqlmap kpartx sleuthkit qemu-utils gcc openvpn wfuzz smbclient mysql-client netcat gobuster -y'",
      "docker exec -it webtop bash -c 'curl https://raw.githubusercontent.com/rapid7/metasploit-omnibus/master/config/templates/metasploit-framework-wrappers/msfupdate.erb > msfinstall && chmod +x msfinstall && ./msfinstall'",
      "docker exec -it webtop bash -c 'git clone https://github.com/offensive-security/exploitdb.git /opt/exploitdb && ln -sf /opt/exploitdb/searchsploit /usr/local/bin/searchsploit'",
      "docker exec -it webtop bash -c 'git clone https://github.com/danielmiessler/SecLists.git'",
      "docker exec -it webtop bash -c 'apt install ruby-full -y && gem install wpscan'",
      "docker exec -it webtop bash -c 'git clone https://github.com/rezasp/joomscan.git && apt install libwww-perl -y'"
    ]
  }

  // Install code-server
  provisioner "remote-exec" {
    inline = [
      "curl -fsSL https://code-server.dev/install.sh | sh",
      "mkdir -p  /root/.config/code-server/",
      "echo 'bind-addr: 0.0.0.0:8080\nauth: password\npassword: yfB4W23G\ncert: false\n' > /root/.config/code-server/config.yaml",
      "systemctl enable --now code-server@$USER"
    ]
  }
}
